// platformDependant.cpp: implementation of the CplatformDependant class.
//
//////////////////////////////////////////////////////////////////////

#include "platformDependant.h"
#include <assert.h>
#ifdef WIN32
#include <process.h>
#else
#include <pthread.h>
#include <errno.h>
#include <dlfcn.h>
#include <stdio.h>
#endif


SThread::SThread():hThread(0){
}



bool SThread::isRunning(){
	bool running = (hThread != 0);
	return running;
}



void SThread::msSleep(unsigned long duration_ms){
#ifdef WIN32
	Sleep(duration_ms);
#else
	usleep(duration_ms);
#endif
}



#ifdef WIN32
void SThread::ThreadLaunch(LPVOID lpParam){
	SThread * threadedObject = (SThread*)lpParam;
	threadedObject->threadMain();
	// thread execution completed - shut it down
	threadedObject->shouldStop = true;
	threadedObject->hThread = NULL;
	_endthread();//CloseHandle(threadedObject->hThread);
	//return 0;
}
#else
void * SThread::ThreadLaunch(void* lpParam){
  	SThread * threadedObject = (SThread*)lpParam;
	threadedObject->threadMain();
	// thread execution completed - shut it down
	threadedObject->shouldStop = true;
	threadedObject->hThread = 0;
	pthread_exit(NULL);
	return 0;
}
#endif



void SThread::start(){
	shouldStop = false;
	if (hThread == 0){
#ifdef WIN32
		hThread = _beginthread(SThread::ThreadLaunch, 0, this);
/*	  hThread = CreateThread( 
        NULL,                        // no security attributes 
        0,                           // use default stack size  
	SThread::ThreadLaunch,       // thread function 
        this,			     // argument to thread function 
        0,                           // use default creation flags 
        &dwThreadId);*/
#else
	  pthread_create( &hThread, NULL, SThread::ThreadLaunch, this);
#endif
	}

	assert(hThread != 0);
}



void SThread::stop(){
	shouldStop = true;
}




SMutex::SMutex(const SString & theName):lockCount(0){
     mutexName = theName;
#ifdef WIN32
	hMutex = CreateMutex( 
		NULL,      // no security attributes
		FALSE,     // initially not owned
		NULL);     // name of mutex
#else 
	pthread_mutex_t  mux = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;//PTHREAD_MUTEX_INITIALIZER;
	hMutex = mux;
#endif
}


SMutex::~SMutex(){
#ifdef WIN32
		CloseHandle(hMutex);
#else
		pthread_mutex_destroy(&hMutex); 
#endif
	}


void SMutex::lock(){
#ifdef WIN32
    int dwWaitResult = WaitForSingleObject( 
        hMutex,       // handle to mutex
        INFINITE); 
 
    switch (dwWaitResult) 
    {
        // The thread got mutex ownership.
        case WAIT_OBJECT_0: 
        case WAIT_ABANDONED:break;
    }
#else
    pthread_mutex_lock(&hMutex);
#endif
	lockCount++;
}


void SMutex::release(){
	lockCount--;
#ifdef WIN32
 ReleaseMutex(hMutex);
#else
  pthread_mutex_unlock( &hMutex );
#endif
}



const SString & SMutex::getName(){
	return mutexName;
}

int SMutex::getLockCount(){
	return lockCount;
}




SEvent::SEvent(const SString &name):eventName(name){
#ifdef WIN32
	event = CreateEvent( 
        NULL,         // no security attributes
        FALSE,         // manual-reset event
        FALSE,         // initial state is nonsignaled
        eventName  // object name
        );
#else
	pthread_mutex_t hM = PTHREAD_MUTEX_INITIALIZER;
	hMutex = hM;
    pthread_cond_t  hC = PTHREAD_COND_INITIALIZER;
	hCondition = hC;
	eventPending = false;
#endif
}


SmutexGuard::SmutexGuard(SMutex &m):mux(m){
//	printf("about to lock %s\n", mux.getName().getString());
	mux.lock();
//	printf("locked %s\n", mux.getName().getString());
}


SmutexGuard::~SmutexGuard(){
//		printf("about to unlock %s\n", mux.getName().getString());
		mux.release();
//		printf("unlocked %s\n", mux.getName().getString());
}


void SEvent::clearPending(){
#ifdef WIN32
	ResetEvent(event);
#else
	eventPending = false;
#endif
}




short SEvent::wait(long timeOutMsec){
	short successVal = 0;
#ifdef WIN32
	  DWORD dwWaitResult = WaitForMultipleObjects( 
      1,   // number of handles in array
      &event,  // array of read-event handles
      TRUE,         // wait until all are signaled
      (timeOutMsec < 0) ? INFINITE : timeOutMsec);

    switch (dwWaitResult) 
    {
        case WAIT_OBJECT_0: 
            successVal = 0;
            break;
		case WAIT_TIMEOUT: 
            successVal = -1;
            break;
        default: 
            successVal = -2;
    }
#else
	pthread_mutex_lock(&hMutex);
	int result;
	if (eventPending == true){
		result = 0;
	} else {
		if (timeOutMsec < 0){
			result = pthread_cond_wait(&hCondition, &hMutex);
		} else {
			struct timeval tvTimeNow;  //Absolute current time
			struct timezone tzTimeZone;  //Timezone
			struct timespec tsTimeOut;   // Input for timedwait
			gettimeofday (&tvTimeNow, &tzTimeZone);
			tsTimeOut.tv_sec = tvTimeNow.tv_sec + (timeOutMsec/1000);
			long timeMsec = (tvTimeNow.tv_usec / 1000) +  (timeOutMsec%1000);
			tsTimeOut.tv_sec += timeMsec / 1000;
			tsTimeOut.tv_nsec = (timeMsec % 1000) * 1000000;
			result = pthread_cond_timedwait (&hCondition, &hMutex, &tsTimeOut);
		}
	}
	eventPending = false;
	pthread_mutex_unlock(&hMutex);
	 switch (result) 
    {
        case 0: // OK 
            successVal = 0;
            break;
		case ETIMEDOUT: 
            successVal = -1;
            break;
        default: 
            successVal = -2;
    }
#endif
	return successVal;
}


void SEvent::signal(){
#ifdef WIN32
	SetEvent(event);
#else
	pthread_mutex_lock(&hMutex);
	eventPending = true;
	pthread_cond_signal(&hCondition);
	pthread_mutex_unlock(&hMutex);
#endif
}



CSharedLibrary::CSharedLibrary(SString & fullyQualifiedLibName){
		fullName = fullyQualifiedLibName;
#ifdef WIN32
		hLibrary = LoadLibrary(fullyQualifiedLibName);
#else
		hLibrary = dlopen(fullyQualifiedLibName, RTLD_NOW);
		if (hLibrary == NULL){
			printf("%s-failed to load\n", fullyQualifiedLibName.getString());
		}
#endif
	}


const SString & CSharedLibrary::getFullName() const{
	return fullName;
}


void * CSharedLibrary::getFunction(SString & decoratedFunctionName){
		void * fptr = NULL;
#ifdef WIN32
		if (hLibrary){
			fptr = GetProcAddress(hLibrary,decoratedFunctionName);
		}
#else
		if (hLibrary){
			fptr = dlsym(hLibrary,decoratedFunctionName);
		}	
#endif
		return fptr;
	}

	CSharedLibrary::~CSharedLibrary(){
#ifdef WIN32
		if (hLibrary){
			FreeLibrary(hLibrary);
		}
#else
		if (hLibrary){
			dlclose(hLibrary);
		}
#endif	
	}
